/*
 * Copyright 1997,1999,2000 Andi Kleen. Subject to the GPL. 
 * $Id: statistics.c,v 1.14 2001/02/02 18:01:23 pb Exp $
 * 19980630 - i18n - Arnaldo Carvalho de Melo <acme@conectiva.com.br> 
 * 19981113 - i18n fixes - Arnaldo Carvalho de Melo <acme@conectiva.com.br> 
 * 19990101 - added net/netstat, -t, -u, -w supprt - Bernd Eckenfels 
 */
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "config.h"
#include "intl.h"

/* #define WARN 1 */

#ifdef WARN
#define UFWARN(x) x
#else
#define UFWARN(x)
#endif

int print_static,f_raw,f_tcp,f_udp,f_sctp,f_unknown = 1;

enum State {
    number = 0, opt_number, i_forward, i_inp_icmp, i_outp_icmp, i_rto_alg,
    MaxState
};

#define normal number

struct entry {
    char *title;
    char *out;
    enum State type;
};

struct statedesc { 
    int indent;
    char *title; 
}; 

struct statedesc states[] = { 
    [number] = { 4, NULL },
    [opt_number] = { 4, NULL }, 
    [i_forward] = { 4, NULL },
    [i_inp_icmp] = { 8, N_("ICMP input histogram:") },
    [i_outp_icmp] = { 8, N_("ICMP output histogram:") },
    [MaxState] = {0},
}; 

static enum State state;

#define I_STATIC (1<<16)	/* static configuration option. */
#define I_TITLE  (1<<17)

/* 
 * XXX check against the snmp mib rfc.
 *
 * Don't mark the first field as translatable! It's a snmp MIB standard.
 * - acme
 */
struct entry Iptab[] =
{
    {"Forwarding", N_("Forwarding is %s"), i_forward | I_STATIC},
    {"DefaultTTL", N_("Default TTL is %lu"), number | I_STATIC},
    {"InReceives", N_("%lu total packets received"), number},
    {"InHdrErrors", N_("%lu with invalid headers"), opt_number},
    {"InAddrErrors", N_("%lu with invalid addresses"), opt_number},
    {"ForwDatagrams", N_("%lu forwarded"), number},
    {"InUnknownProtos", N_("%lu with unknown protocol"), opt_number},
    {"InDiscards", N_("%lu incoming packets discarded"), number},
    {"InDelivers", N_("%lu incoming packets delivered"), number},
    {"OutRequests", N_("%lu requests sent out"), number},	/*? */
    {"OutDiscards", N_("%lu outgoing packets dropped"), opt_number},
    {"OutNoRoutes", N_("%lu dropped because of missing route"), opt_number},
    {"ReasmTimeout", N_("%lu fragments dropped after timeout"), opt_number},
    {"ReasmReqds", N_("%lu reassemblies required"), opt_number},	/* ? */
    {"ReasmOKs", N_("%lu packets reassembled ok"), opt_number},
    {"ReasmFails", N_("%lu packet reassembles failed"), opt_number},
    {"FragOKs", N_("%lu fragments received ok"), opt_number},
    {"FragFails", N_("%lu fragments failed"), opt_number},
    {"FragCreates", N_("%lu fragments created"), opt_number}
};

struct entry Ip6tab[] =
{
    {"Ip6InReceives", N_("%u total packets received"), number},
    {"Ip6InHdrErrors", N_("%u with invalid headers"), opt_number},
    {"Ip6InTooBigErrors", N_("%u with packets too big"), opt_number},
    {"Ip6InNoRoutes", N_("%u incoming packets with no route"), opt_number},
    {"Ip6InAddrErrors", N_("%u with invalid addresses"), opt_number},
    {"Ip6InUnknownProtos", N_("%u with unknown protocol"), opt_number},
    {"Ip6InTruncatedPkts", N_("%u with truncated packets"), opt_number},
    {"Ip6InDiscards", N_("%u incoming packets discarded"), number},
    {"Ip6InDelivers", N_("%u incoming packets delivered"), number},
    {"Ip6OutForwDatagrams", N_("%u forwarded"), number},
    {"Ip6OutRequests", N_("%u requests sent out"), number},     /*? */
    {"Ip6OutDiscards", N_("%u outgoing packets dropped"), opt_number},
    {"Ip6OutNoRoutes", N_("%u dropped because of missing route"), opt_number},
    {"Ip6ReasmTimeout", N_("%u fragments dropped after timeout"), opt_number},
    {"Ip6ReasmReqds", N_("%u reassemblies required"), opt_number}, /* ? */
    {"Ip6ReasmOKs", N_("%u packets reassembled ok"), opt_number},
    {"Ip6ReasmFails", N_("%u packet reassembles failed"), opt_number},
    {"Ip6FragOKs", N_("%u fragments received ok"), opt_number},
    {"Ip6FragFails", N_("%u fragments failed"), opt_number},
    {"Ip6FragCreates", N_("%u fragments created"), opt_number},
    {"Ip6InMcastPkts", N_("%u incoming multicast packets"), opt_number},
    {"Ip6OutMcastPkts", N_("%u outgoing multicast packets"), opt_number}
};

struct entry Icmptab[] =
{
    {"InMsgs", N_("%lu ICMP messages received"), number},
    {"InErrors", N_("%lu input ICMP message failed."), number},
    {"InDestUnreachs", N_("destination unreachable: %lu"), i_inp_icmp | I_TITLE},
    {"InTimeExcds", N_("timeout in transit: %lu"), i_inp_icmp | I_TITLE},
    {"InParmProbs", N_("wrong parameters: %lu"), i_inp_icmp | I_TITLE},	/*? */
    {"InSrcQuenchs", N_("source quenches: %lu"), i_inp_icmp | I_TITLE},
    {"InRedirects", N_("redirects: %lu"), i_inp_icmp | I_TITLE},
    {"InEchos", N_("echo requests: %lu"), i_inp_icmp | I_TITLE},
    {"InEchoReps", N_("echo replies: %lu"), i_inp_icmp | I_TITLE},
    {"InTimestamps", N_("timestamp request: %lu"), i_inp_icmp | I_TITLE},
    {"InTimestampReps", N_("timestamp reply: %lu"), i_inp_icmp | I_TITLE},
    {"InAddrMasks", N_("address mask request: %lu"), i_inp_icmp | I_TITLE},	/*? */
    {"InAddrMaskReps", N_("address mask replies: %lu"), i_inp_icmp | I_TITLE},	/*? */
    {"OutMsgs", N_("%lu ICMP messages sent"), number},
    {"OutErrors", N_("%lu ICMP messages failed"), number},
    {"OutDestUnreachs", N_("destination unreachable: %lu"), i_outp_icmp | I_TITLE},
    {"OutTimeExcds", N_("time exceeded: %lu"), i_outp_icmp | I_TITLE},
    {"OutParmProbs", N_("wrong parameters: %lu"), i_outp_icmp | I_TITLE},	/*? */
    {"OutSrcQuenchs", N_("source quench: %lu"), i_outp_icmp | I_TITLE},
    {"OutRedirects", N_("redirect: %lu"), i_outp_icmp | I_TITLE},
    {"OutEchos", N_("echo request: %lu"), i_outp_icmp | I_TITLE},
    {"OutEchoReps", N_("echo replies: %lu"), i_outp_icmp | I_TITLE},
    {"OutTimestamps", N_("timestamp requests: %lu"), i_outp_icmp | I_TITLE},
    {"OutTimestampReps", N_("timestamp replies: %lu"), i_outp_icmp | I_TITLE},
    {"OutAddrMasks", N_("address mask requests: %lu"), i_outp_icmp | I_TITLE},
    {"OutAddrMaskReps", N_("address mask replies: %lu"), i_outp_icmp | I_TITLE},
};

struct entry Icmp6tab[] =
{
    {"Icmp6InMsgs", N_("%u ICMP messages received"), number},
    {"Icmp6InErrors", N_("%u input ICMP message failed."), number},
    {"Icmp6InDestUnreachs", N_("destination unreachable: %u"), i_inp_icmp | I_TITLE},
    {"Icmp6InPktTooBigs", N_("packets too big: %u"), i_inp_icmp | I_TITLE},
    {"Icmp6InTimeExcds", N_("received ICMPv6 time exceeded: %u"), i_inp_icmp | I_TITLE},
    {"Icmp6InParmProblems", N_("parameter problem: %u"), i_inp_icmp | I_TITLE},
    {"Icmp6InEchos", N_("echo requests: %u"), i_inp_icmp | I_TITLE},
    {"Icmp6InEchoReplies", N_("echo replies: %u"), i_inp_icmp | I_TITLE},
    {"Icmp6InGroupMembQueries", N_("group member queries: %u"), i_inp_icmp | I_TITLE},
    {"Icmp6InGroupMembResponses", N_("group member responses: %u"), i_inp_icmp | I_TITLE},
    {"Icmp6InGroupMembReductions", N_("group member reductions: %u"), i_inp_icmp | I_TITLE},
    {"Icmp6InRouterSolicits", N_("router solicits: %u"), i_inp_icmp | I_TITLE},
    {"Icmp6InRouterAdvertisements", N_("router advertisement: %u"), i_inp_icmp | I_TITLE},
    {"Icmp6InNeighborSolicits", N_("neighbour solicits: %u"), i_inp_icmp | I_TITLE},
    {"Icmp6InNeighborAdvertisements", N_("neighbour advertisement: %u"), i_inp_icmp | I_TITLE},
    {"Icmp6InRedirects", N_("redirects: %u"), i_inp_icmp | I_TITLE},
    {"Icmp6OutMsgs", N_("%u ICMP messages sent"), number},
    {"Icmp6OutDestUnreachs", N_("destination unreachable: %u"), i_outp_icmp | I_TITLE},
    {"Icmp6OutPktTooBigs", N_("packets too big: %u"), i_outp_icmp | I_TITLE},
    {"Icmp6OutTimeExcds", N_("sent ICMPv6 time exceeded: %u"), i_outp_icmp | I_TITLE},
    {"Icmp6OutParmProblems", N_("parameter problem: %u"), i_outp_icmp | I_TITLE},
    {"Icmp6OutEchos", N_("echo requests: %u"), i_outp_icmp | I_TITLE},
    {"Icmp6OutEchoReplies", N_("echo replies: %u"), i_outp_icmp | I_TITLE},
    {"Icmp6OutGroupMembQueries", N_("group member queries: %u"), i_outp_icmp | I_TITLE},
    {"Icmp6OutGroupMembResponses", N_("group member responses: %u"), i_outp_icmp | I_TITLE},
    {"Icmp6OutGroupMembReductions", N_("group member reductions: %u"), i_outp_icmp | I_TITLE},
    {"Icmp6OutRouterSolicits", N_("router solicits: %u"), i_outp_icmp | I_TITLE},
    {"Icmp6OutRouterAdvertisements ", N_("router advertisement: %u"), i_outp_icmp | I_TITLE},
    {"Icmp6OutNeighborSolicits", N_("neighbor solicits: %u"), i_outp_icmp | I_TITLE},
    {"Icmp6OutNeighborAdvertisements", N_("neighbor advertisements: %u"), i_outp_icmp | I_TITLE},
    {"Icmp6OutRedirects", N_("redirects: %u"), i_outp_icmp | I_TITLE},
};

struct entry Tcptab[] =
{
    {"RtoAlgorithm", N_("RTO algorithm is %s"), i_rto_alg | I_STATIC},
    {"RtoMin", "", number},
    {"RtoMax", "", number},
    {"MaxConn", "", number},
    {"ActiveOpens", N_("%lu active connections openings"), number},
    {"PassiveOpens", N_("%lu passive connection openings"), number},
    {"AttemptFails", N_("%lu failed connection attempts"), number},
    {"EstabResets", N_("%lu connection resets received"), number},
    {"CurrEstab", N_("%lu connections established"), number},
    {"InSegs", N_("%lu segments received"), number},
    {"OutSegs", N_("%lu segments send out"), number},
    {"RetransSegs", N_("%lu segments retransmited"), number},
    {"InErrs", N_("%lu bad segments received."), number},
    {"OutRsts", N_("%lu resets sent"), number},
};

struct entry Udptab[] =
{
    {"InDatagrams", N_("%lu packets received"), number},
    {"NoPorts", N_("%lu packets to unknown port received."), number},
    {"InErrors", N_("%lu packet receive errors"), number},
    {"OutDatagrams", N_("%lu packets sent"), number},
};

struct entry Udp6tab[] =
{
    {"Udp6InDatagrams", N_("%u packets received"), number},
    {"Udp6NoPorts", N_("%u packets to unknown port received."), number},
    {"Udp6InErrors", N_("%u packet receive errors"), number},
    {"Udp6OutDatagrams", N_("%u packets sent"), number},
};

struct entry Tcpexttab[] =
{
    {"SyncookiesSent", N_("%lu SYN cookies sent"), opt_number},
    {"SyncookiesRecv", N_("%lu SYN cookies received"), opt_number},
    {"SyncookiesFailed", N_("%lu invalid SYN cookies received"), opt_number},

    { "EmbryonicRsts", N_("%lu resets received for embryonic SYN_RECV sockets"),
      opt_number },  
    { "PruneCalled", N_("%lu packets pruned from receive queue because of socket"
			" buffer overrun"), opt_number },  
    /* obsolete: 2.2.0 doesn't do that anymore */
    { "RcvPruned", N_("%lu packets pruned from receive queue"), opt_number },
    { "OfoPruned", N_("%lu packets dropped from out-of-order queue because of"
		      " socket buffer overrun"), opt_number }, 
    { "OutOfWindowIcmps", N_("%lu ICMP packets dropped because they were "
			     "out-of-window"), opt_number }, 
    { "LockDroppedIcmps", N_("%lu ICMP packets dropped because"
			     " socket was locked"), opt_number },
    { "TW", N_("%lu TCP sockets finished time wait in fast timer"), opt_number },
    { "TWRecycled", N_("%lu time wait sockets recycled by time stamp"), opt_number }, 
    { "TWKilled", N_("%lu TCP sockets finished time wait in slow timer"), opt_number },
    { "PAWSPassive", N_("%lu passive connections rejected because of"
			" time stamp"), opt_number },
    { "PAWSActive", N_("%lu active connections rejected because of "
		       "time stamp"), opt_number },
    { "PAWSEstab", N_("%lu packets rejects in established connections because of"
		      " timestamp"), opt_number },
    { "DelayedACKs", N_("%lu delayed acks sent"), opt_number },
    { "DelayedACKLocked", N_("%lu delayed acks further delayed because of"
			     " locked socket"), opt_number },
    { "DelayedACKLost", N_("Quick ack mode was activated %lu times"), opt_number },
    { "ListenOverflows", N_("%lu times the listen queue of a socket overflowed"),
      opt_number },
    { "ListenDrops", N_("%lu SYNs to LISTEN sockets ignored"), opt_number },
    { "TCPPrequeued", N_("%lu packets directly queued to recvmsg prequeue."), 
      opt_number },
    { "TCPDirectCopyFromBacklog", N_("%lu packets directly received"
				     " from backlog"), opt_number },
    { "TCPDirectCopyFromPrequeue", N_("%lu packets directly received"
				      " from prequeue"), opt_number },
    { "TCPPrequeueDropped", N_("%lu packets dropped from prequeue"), opt_number },
    { "TCPHPHits", N_("%lu packets header predicted"), number },
    { "TCPHPHitsToUser", N_("%lu packets header predicted and "
			    "directly queued to user"), opt_number },
    { "SockMallocOOM", N_("Ran %lu times out of system memory during " 
			  "packet sending"), opt_number }, 
};

struct entry Sctptab[] =
{
    {"SctpCurrEstab", N_("%u Current Associations"), number},
    {"SctpActiveEstabs", N_("%u Active Associations"), number},
    {"SctpPassiveEstabs", N_("%u Passive Associations"), number},
    {"SctpAborteds", N_("%u Number of Aborteds "), number},
    {"SctpShutdowns", N_("%u Number of Graceful Terminations"), number},
    {"SctpOutOfBlues", N_("%u Number of Out of Blue packets"), number},
    {"SctpChecksumErrors", N_("%u Number of Packets with invalid Checksum"), number},
    {"SctpOutCtrlChunks", N_("%u Number of control chunks sent"), number},
    {"SctpOutOrderChunks", N_("%u Number of ordered chunks sent"), number},
    {"SctpOutUnorderChunks", N_("%u Number of Unordered chunks sent"), number},
    {"SctpInCtrlChunks", N_("%u Number of control chunks received"), number},
    {"SctpInOrderChunks", N_("%u Number of ordered chunks received"), number},
    {"SctpInUnorderChunks", N_("%u Number of Unordered chunks received"), number},
    {"SctpFragUsrMsgs", N_("%u Number of messages fragmented"), number},
    {"SctpReasmUsrMsgs", N_("%u Number of messages reassembled "), number},
    {"SctpOutSCTPPacks", N_("%u Number of SCTP packets sent"), number},
    {"SctpInSCTPPacks", N_("%u Number of SCTP packets received"), number},
};

struct tabtab {
    char *title;
    struct entry *tab;
    size_t size;
    int *flag; 
};

struct tabtab snmptabs[] =
{
    {"Ip", Iptab, sizeof(Iptab), &f_raw},
    {"Icmp", Icmptab, sizeof(Icmptab), &f_raw},
    {"Tcp", Tcptab, sizeof(Tcptab), &f_tcp},
    {"Udp", Udptab, sizeof(Udptab), &f_udp},
    {"Sctp", Sctptab, sizeof(Sctptab), &f_sctp},
    {"TcpExt", Tcpexttab, sizeof(Tcpexttab), &f_tcp},
    {NULL}
};

struct tabtab snmp6tabs[] =
{
    {"Ip6", Ip6tab, sizeof(Ip6tab), &f_raw},
    {"Icmp6", Icmp6tab, sizeof(Icmp6tab), &f_raw},
    {"Udp6", Udp6tab, sizeof(Udp6tab), &f_udp},
    {"Tcp6", Tcptab, sizeof(Tcptab), &f_tcp},
    {NULL}
};

/* XXX IGMP */

int cmpentries(const void *a, const void *b)
{
    return strcmp(((struct entry *) a)->title, ((struct entry *) b)->title);
}

void printval(struct tabtab *tab, char *title, signed long val)
{
    struct entry *ent = NULL, key;
    int type;
    char buf[512];

    key.title = title;
	if (tab->tab) 
	    ent = bsearch(&key, tab->tab, tab->size / sizeof(struct entry),
			  sizeof(struct entry), cmpentries);
    if (!ent) {			/* try our best */
	printf("%*s%s: %lu\n", states[state].indent, "", title, val);
	return;
    }
    type = ent->type;
    if (type & I_STATIC) {
	type &= ~I_STATIC;
	if (!print_static)
	    return;
    }
    if (*ent->out == '\0')
	return;

    if (type & I_TITLE) {
	type &= ~I_TITLE;
	if (state != type)
	    printf("%*s%s\n", states[state].indent, "", _(states[type].title));
    }
    buf[0] = '\0';
    switch (type) {
    case opt_number:
	if (val == 0) 
	    break;
	/*FALL THOUGH*/
    case number:
	snprintf(buf, sizeof(buf), _(ent->out), val);
	break;
    case i_forward:
	type = normal;
	snprintf(buf, sizeof(buf), _(ent->out), val == 2 ? _("enabled") : _("disabled"));
	break;
    case i_outp_icmp:
    case i_inp_icmp:
	if (val > 0)
	    snprintf(buf, sizeof(buf), _(ent->out), val);
	break;
    case i_rto_alg:		/* XXXX */
	break;
    default:
	abort();
    }
    if (buf[0])
	printf("%*s%s\n", states[type].indent, "", buf);

    state = type;
}

struct tabtab *newtable(struct tabtab *tabs, char *title)
{
    struct tabtab *t;
	static struct tabtab dummytab;
	
    for (t = tabs; t->title; t++) {
		if (!strcmp(title, t->title)) {
	    	if (*(t->flag))
				printf("%s:\n", _(title));
		    state = normal;
	   		return t;
		}
	}
	if (!f_unknown) 
		return NULL; 
	printf("%s:\n", _(title));
	dummytab.title = title;
	dummytab.flag = &f_unknown; 
	return &dummytab;
}

void process_fd(FILE *f, int all, char *filter)
{
    char buf1[8192], buf2[8192];
    char *sp, *np, *p;
    while (fgets(buf1, sizeof buf1, f)) {
	int endflag;
	struct tabtab *tab;

	if (!fgets(buf2, sizeof buf2, f))
	    break;
	sp = strchr(buf1, ':');
	np = strchr(buf2, ':');
	if (!np || !sp)
	    goto formaterr;
	*sp = '\0';

	if (!all) {
		if (strncmp(buf1, filter, strlen(filter))) {
			continue;
		}
	}

	tab = newtable(snmptabs, buf1);
	if (tab == NULL) {
		printf("unknown %s\n", buf1);
		continue;
	}
	np++;
	sp++;

	endflag = 0;
	while (!endflag) {
	    sp += strspn(sp, " \t\n"); 
	    np += strspn(np, " \t\n"); 
	    /*if (*np == '\0') goto formaterr; */

	    p = sp+strcspn(sp, " \t\n");
	    if (*p == '\0')
		endflag = 1;
	    *p = '\0';

	    if (*sp != '\0' && *(tab->flag)) 	
		printval(tab, sp, strtoul(np, &np, 10));

	    sp = p + 1;
	}
    }
  return;
  
formaterr:
  perror(_("error parsing /proc/net/snmp"));
  return;
}

/* Process a file with name-value lines (like /proc/net/sctp/snmp) */
void process_fd2(FILE *f, const char *filename)
{
    char buf1[1024];
    char *sp;
    struct tabtab *tab;
    
    tab = newtable(snmptabs, "Sctp");
    
    while (fgets(buf1, sizeof buf1, f)) {
	sp = buf1 + strcspn(buf1, " \t\n");
	if (!sp)
	    goto formaterr;
	*sp = '\0';
    	sp++;
	
	sp += strspn(sp, " \t\n"); 

	if (*sp != '\0' && *(tab->flag)) 	
	    printval(tab, buf1, strtoul(sp, 0, 10));
    }
  return;
  
formaterr:
  fprintf(stderr,_("error parsing %s\n"), filename);
  return;
}

void cpytitle(char *original, char *new)
{
	char *ptr = original;
	while(*ptr != '6' && *ptr != '\0') {
		*new = *ptr;
		new++;
		ptr++;
	}
	*new = *ptr;
	new++;
	*new = '\0';
}

void process6_fd(FILE *f)
{
	char buf1[1024],buf2[50],buf3[1024];
	unsigned long val;
	struct tabtab *tab = NULL;
	int cpflg = 0;

	while (fgets(buf1, sizeof buf1, f)) {
		sscanf(buf1, "%s %lu", buf2, &val);
		if(!cpflg) {
			cpytitle(buf2, buf3);
			tab = newtable(snmp6tabs, buf3);
			cpflg = 1;
		}
		if(!strstr(buf2, buf3)) {
			cpytitle(buf2, buf3);
			tab = newtable(snmp6tabs, buf3);
		}
		printval(tab, buf2, val);
	}
}

void parsesnmp(int flag_raw, int flag_tcp, int flag_udp, int flag_sctp)
{
    FILE *f;

    f_raw = flag_raw; f_tcp = flag_tcp; f_udp = flag_udp, f_sctp = flag_sctp;
    
    f = fopen("/proc/net/snmp", "r");
    if (!f) {
	perror(_("cannot open /proc/net/snmp"));
	return;
    }
	process_fd(f, 1, NULL);

    if (ferror(f))
	perror("/proc/net/snmp");

    fclose(f);

    f = fopen("/proc/net/netstat", "r");

    if (f) {
		process_fd(f, 1, NULL);

        if (ferror(f))
	    perror("/proc/net/netstat");
    
        fclose(f);
    }

    f = fopen("/proc/net/sctp/snmp", "r");
    if (f) {
	process_fd2(f,"/proc/net/sctp/snmp");
	if (ferror(f))
	    perror("/proc/net/sctp/snmp");

	fclose(f);
    }


    return;
}

void parsesnmp6(int flag_raw, int flag_tcp, int flag_udp, int flag_sctp)
{
	FILE *f;

	f_raw = flag_raw; f_tcp = flag_tcp; f_udp = flag_udp, f_sctp = flag_sctp;

	f = fopen("/proc/net/snmp6", "r");
	if (!f) {
		perror(_("cannot open /proc/net/snmp6"));
		return;
	}
	process6_fd(f);
	if (ferror(f))
		perror("/proc/net/snmp6");

	fclose(f);
	f = fopen("/proc/net/snmp", "r");
	if (!f) {
		perror(_("cannot open /proc/net/snmp"));
		return;
	}
	process_fd(f, 0, "Tcp");
	if (ferror(f))
		perror("/proc/net/snmp");
	fclose(f);
}

void inittab(void)
{
    struct tabtab *t;

    /* we sort at runtime because I'm lazy ;) */
    for (t = snmptabs; t->title; t++)
	qsort(t->tab, t->size / sizeof(struct entry),
	      sizeof(struct entry), cmpentries);
}

void inittab6(void)
{
	struct tabtab *t;

	for (t = snmp6tabs; t->title; t++)
		qsort(t->tab, t->size / sizeof(struct entry),
			sizeof(struct entry), cmpentries);
}

